/* Name: _matrixio.h
 * Purpose: header file for _matrixio.c
 * Author: Ken McIvor <mcivken@iit.edu>
 *
 * Copyright 2003-2004 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */


#ifndef _MATRIXIO_H
#define _MATRIXIO_H

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif



#define _MIO_BUFFSTART 32768
#define _MIO_BUFFINCR  16384
#define _MIO_DATASTART 2048
#define _MIO_DATAINCR  1024


#define _MERR_OKAY 0
#define _MERR_IBUFFMEM -100
#define _MERR_BUFFMEM  -101
#define _MERR_IDATAMEM -102
#define _MERR_DATAMEM  -103
#define _MERR_CONVERT  -200
#define _MERR_FEWCOL   -300
#define _MERR_MANYCOL  -301


#define _matrixio_API_pointers 4

#define _matrixio_pyRead_NUM 0
#define _matrixio_pyRead_RETURN PyObject *
#define _matrixio_pyRead_PROTO (PyObject *, PyObject *)

#define _matrixio_pyWrite_NUM 1
#define _matrixio_pyWrite_RETURN PyObject *
#define _matrixio_pyWrite_PROTO (PyObject *, PyObject *)

#define _matrixio_read_NUM 2
#define _matrixio_read_RETURN int
#define _matrixio_read_PROTO (FILE *, double **, int *, int *)

#define _matrixio_write_NUM 3
#define _matrixio_write_RETURN int
#define _matrixio_write_PROTO (FILE *, PyArrayObject *)



#ifdef MATRIXIO_MODULE

static _matrixio_pyRead_RETURN matrixio_pyRead _matrixio_pyRead_PROTO;
static _matrixio_pyWrite_RETURN matrixio_pyWrite _matrixio_pyWrite_PROTO;
static _matrixio_read_RETURN matrixio_read _matrixio_read_PROTO;
static _matrixio_write_RETURN matrixio_write _matrixio_write_PROTO;

#else

static void **_matrixio_API;

#define matrixio_pyRead \
 (*(_matrixio_pyRead_RETURN (*)_matrixio_pyRead_PROTO) \
  _matrixio_API[_matrixio_pyRead_NUM])

#define matrixio_pyWrite \
 (*(_matrixio_pyWrite_RETURN (*)_matrixio_pyWrite_PROTO) \
  _matrixio_API[_matrixio_pyWrite_NUM])

#define matrixio_read \
 (*(_matrixio_read_RETURN (*)_matrixio_read_PROTO) \
  _matrixio_API[_matrixio_read_NUM])

#define matrixio_write \
 (*(_matrixio_write_RETURN (*)_matrixio_write_PROTO) \
  _matrixio_API[_matrixio_write_NUM])

#define import__matrixio() \
{ \
  PyObject *module = PyImport_ImportModule("xdp.io._matrixio"); \
  if (module != NULL) { \
    PyObject *module_dict = PyModule_GetDict(module); \
    PyObject *c_api_object = PyDict_GetItemString(module_dict, "_C_API"); \
    if (PyCObject_Check(c_api_object)) { \
      _matrixio_API = (void **)PyCObject_AsVoidPtr(c_api_object); \
    } \
  } \
}

#endif /* MATRIXIO_MODULE */
 

#ifdef __cplusplus
}
#endif

#endif /* _MATRIXIO_H */
